import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import (
    BaseUserGroupFields,
    MessagesChat,
    MessagesChatFull,
    MessagesGetConversationById,
    MessagesGetConversationByIdExtended,
    MessagesGetConversationMembers,
    MessagesLastActivity,
    MessagesLongpollParams,
    MessagesPinnedMessage,
    UsersFields,
)
from vkbottle_types.responses.base import (
    BaseBoolResponse,
    BaseOkResponse,
    BaseOkResponseModel,
)
from vkbottle_types.responses.base_response import DictResponse
from vkbottle_types.responses.messages import *  # noqa: F401,F403  # type: ignore


class MessagesCategory(BaseCategory):
    async def add_chat_user(
        self,
        chat_id: int,
        user_id: int | None = None,
        visible_messages_count: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.addChatUser()`

        :param chat_id: Chat ID.
        :param user_id: ID of the user to be added to the chat.
        :param visible_messages_count:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.addChatUser", params)
        model = BaseOkResponse
        return model(**response).response

    async def add_chat_users(
        self,
        chat_id: int | None = None,
        visible_messages_count: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesAddChatUsersResponseModel:
        """Method `messages.addChatUsers()`

        :param chat_id:
        :param visible_messages_count:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.addChatUsers", params)
        model = MessagesAddChatUsersResponse
        return model(**response).response

    async def allow_messages_from_group(
        self,
        group_id: int,
        key: str | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.allowMessagesFromGroup()`

        :param group_id: Group ID.
        :param key:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.allowMessagesFromGroup", params)
        model = BaseOkResponse
        return model(**response).response

    async def create_chat(
        self,
        group_id: int | None = None,
        title: str | None = None,
        user_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> MessagesCreateChatWithPeerIdsResponseModel:
        """Method `messages.createChat()`

        :param group_id:
        :param title: Chat title.
        :param user_ids: IDs of the users to be added to the chat.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.createChat", params)
        model = MessagesCreateChatWithPeerIdsResponse
        return model(**response).response

    async def delete(
        self,
        cmids: list[int] | None = None,
        delete_for_all: bool | None = None,
        group_id: int | None = None,
        message_ids: list[int] | None = None,
        peer_id: int | None = None,
        reason: int | None = None,
        spam: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[MessagesDeleteFullResponseItem]:
        """Method `messages.delete()`

        :param cmids: Conversation message IDs.
        :param delete_for_all: '1' - delete message for for all.
        :param group_id: Group ID (for group messages with user access token)
        :param message_ids: Message IDs.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param reason: Reason for spam
        :param spam: '1' - to mark message as spam.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.delete", params)
        model = MessagesDeleteFullResponse
        return model(**response).response

    async def delete_chat_photo(
        self,
        chat_id: int,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesDeleteChatPhotoResponseModel:
        """Method `messages.deleteChatPhoto()`

        :param chat_id: Chat ID.
        :param group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.deleteChatPhoto", params)
        model = MessagesDeleteChatPhotoResponse
        return model(**response).response

    async def delete_conversation(
        self,
        group_id: int | None = None,
        peer_id: int | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesDeleteConversationResponseModel:
        """Method `messages.deleteConversation()`

        :param group_id: Group ID (for group messages with user access token)
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param user_id: User ID. To clear a chat history use 'chat_id'
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.deleteConversation", params)
        model = MessagesDeleteConversationResponse
        return model(**response).response

    async def delete_reaction(
        self,
        cmid: int,
        peer_id: int,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `messages.deleteReaction()`

        :param cmid:
        :param peer_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.deleteReaction", params)
        model = BaseBoolResponse
        return model(**response).response

    async def deny_messages_from_group(
        self,
        group_id: int,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.denyMessagesFromGroup()`

        :param group_id: Group ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.denyMessagesFromGroup", params)
        model = BaseOkResponse
        return model(**response).response

    async def edit(
        self,
        peer_id: int,
        attachment: str | None = None,
        cmid: int | None = None,
        disable_mentions: bool | None = None,
        dont_parse_links: bool | None = None,
        group_id: int | None = None,
        keep_forward_messages: bool | None = None,
        keep_snippets: bool | None = None,
        keyboard: str | None = None,
        lat: float | None = None,
        long: float | None = None,
        message: str | None = None,
        message_id: int | None = None,
        template: str | None = None,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `messages.edit()`

        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, 'wall' - wall post, '<owner_id>' - ID of the media attachment owner. '<media_id>' - media attachment ID. Example: "photo100172_166443618"
        :param cmid:
        :param disable_mentions:
        :param dont_parse_links:
        :param group_id: Group ID (for group messages with user access token)
        :param keep_forward_messages: '1' - to keep forwarded, messages.
        :param keep_snippets: '1' - to keep attached snippets.
        :param keyboard:
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param message_id:
        :param template:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.edit", params)
        model = BaseBoolResponse
        return model(**response).response

    async def edit_chat(
        self,
        chat_id: int,
        title: str | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.editChat()`

        :param chat_id: Chat ID.
        :param title: New title of the chat.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.editChat", params)
        model = BaseOkResponse
        return model(**response).response

    @typing.overload
    async def get_by_conversation_message_id(
        self,
        conversation_message_ids: list[int],
        peer_id: int,
        extended: typing.Literal[True],
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetByConversationMessageIdExtendedResponseModel: ...

    @typing.overload
    async def get_by_conversation_message_id(
        self,
        conversation_message_ids: list[int],
        peer_id: int,
        extended: typing.Literal[False] | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetByConversationMessageIdResponseModel: ...

    async def get_by_conversation_message_id(
        self,
        conversation_message_ids: list[int],
        peer_id: int,
        extended: bool | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetByConversationMessageIdExtendedResponseModel | MessagesGetByConversationMessageIdResponseModel:
        """Method `messages.getByConversationMessageId()`

        :param conversation_message_ids: Conversation message IDs.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getByConversationMessageId", params)
        model = self.get_model(
            ((("extended",), MessagesGetByConversationMessageIdExtendedResponse),),
            default=MessagesGetByConversationMessageIdResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_by_id(
        self,
        extended: typing.Literal[True],
        cmids: list[int] | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        message_ids: list[int] | None = None,
        peer_id: int | None = None,
        preview_length: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetByIdExtendedResponseModel: ...

    @typing.overload
    async def get_by_id(
        self,
        extended: typing.Literal[False] | None = None,
        cmids: list[int] | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        message_ids: list[int] | None = None,
        peer_id: int | None = None,
        preview_length: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetByIdResponseModel: ...

    async def get_by_id(
        self,
        extended: bool | None = None,
        cmids: list[int] | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        message_ids: list[int] | None = None,
        peer_id: int | None = None,
        preview_length: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetByIdExtendedResponseModel | MessagesGetByIdResponseModel:
        """Method `messages.getById()`

        :param extended: Information whether the response should be extended
        :param cmids:
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        :param message_ids: Message IDs.
        :param peer_id:
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getById", params)
        model = self.get_model(
            ((("extended",), MessagesGetByIdExtendedResponse),),
            default=MessagesGetByIdResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_chat(
        self,
        chat_ids: list[int],
        fields: list[UsersFields],
        chat_id: int | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesChatFull": ...

    @typing.overload
    async def get_chat(
        self,
        chat_ids: list[int] | None = None,
        fields: list[UsersFields] | None = None,
        chat_id: int | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> list[MessagesChat]: ...

    @typing.overload
    async def get_chat(
        self,
        chat_ids: list[int],
        fields: list[UsersFields],
        chat_id: int | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> list[MessagesChatFull]: ...

    @typing.overload
    async def get_chat(
        self,
        chat_ids: list[int] | None = None,
        fields: list[UsersFields] | None = None,
        chat_id: int | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesChat": ...

    async def get_chat(
        self,
        chat_ids: list[int] | None = None,
        fields: list[UsersFields] | None = None,
        chat_id: int | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesChat | list[MessagesChat] | list[MessagesChatFull] | MessagesChatFull":
        """Method `messages.getChat()`

        :param chat_ids: Chat IDs.
        :param fields: Profile fields to return.
        :param chat_id: Chat ID.
        :param name_case: Case for declension of user name and surname: 'nom' - nominative (default), 'gen' - genitive , 'dat' - dative, 'acc' - accusative , 'ins' - instrumental , 'abl' - prepositional
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getChat", params)
        model = self.get_model(
            (
                (("chat_ids",), MessagesGetChatFieldsResponse),
                (("fields",), MessagesGetChatFieldsResponse),
            ),
            default=MessagesGetChatResponse,
            params=params,
        )
        return model(**response).response

    async def get_chat_preview(
        self,
        fields: list[UsersFields] | None = None,
        link: str | None = None,
        peer_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetChatPreviewResponseModel:
        """Method `messages.getChatPreview()`

        :param fields: Profile fields to return.
        :param link: Invitation link.
        :param peer_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getChatPreview", params)
        model = MessagesGetChatPreviewResponse
        return model(**response).response

    async def get_conversation_members(
        self,
        peer_id: int,
        count: int | None = None,
        extended: bool | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        member_ids: list[int] | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesGetConversationMembers":
        """Method `messages.getConversationMembers()`

        :param peer_id: Peer ID.
        :param count:
        :param extended: Extended flag
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        :param member_ids:
        :param offset:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getConversationMembers", params)
        model = MessagesGetConversationMembersResponse
        return model(**response).response

    async def get_conversations(
        self,
        count: int | None = None,
        extended: bool | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        filter: str | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        start_message_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetConversationsResponseModel:
        """Method `messages.getConversations()`

        :param count: Number of conversations to return.
        :param extended: '1' - return extra information about users and communities
        :param fields: Profile and communities fields to return.
        :param filter: Filter to apply: 'all' - all conversations, 'unread' - conversations with unread messages, 'important' - conversations, marked as important (only for community messages), 'unanswered' - conversations, marked as unanswered (only for community messages)
        :param group_id: Group ID (for group messages with group access token)
        :param offset: Offset needed to return a specific subset of conversations.
        :param start_message_id: ID of the message from what to return dialogs.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getConversations", params)
        model = MessagesGetConversationsResponse
        return model(**response).response

    @typing.overload
    async def get_conversations_by_id(
        self,
        peer_ids: list[int],
        extended: typing.Literal[True],
        fields: list[BaseUserGroupFields] | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesGetConversationByIdExtended": ...

    @typing.overload
    async def get_conversations_by_id(
        self,
        peer_ids: list[int],
        extended: typing.Literal[False] | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesGetConversationById": ...

    async def get_conversations_by_id(
        self,
        peer_ids: list[int],
        extended: bool | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesGetConversationById | MessagesGetConversationByIdExtended":
        """Method `messages.getConversationsById()`

        :param peer_ids: Destination IDs. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param extended: Return extended properties
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getConversationsById", params)
        model = self.get_model(
            ((("extended",), MessagesGetConversationsByIdExtendedResponse),),
            default=MessagesGetConversationsByIdResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_history(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        rev: int | None = None,
        start_message_id: int | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetHistoryExtendedResponseModel: ...

    @typing.overload
    async def get_history(
        self,
        extended: typing.Literal[False] | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        rev: int | None = None,
        start_message_id: int | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetHistoryResponseModel: ...

    async def get_history(
        self,
        extended: bool | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        rev: int | None = None,
        start_message_id: int | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetHistoryResponseModel | MessagesGetHistoryExtendedResponseModel:
        """Method `messages.getHistory()`

        :param extended: Information whether the response should be extended
        :param count: Number of messages to return.
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        :param offset: Offset needed to return a specific subset of messages.
        :param peer_id:
        :param rev: Sort order: '1' - return messages in chronological order. '0' - return messages in reverse chronological order.
        :param start_message_id: Starting message ID from which to return history.
        :param user_id: ID of the user whose message history you want to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getHistory", params)
        model = self.get_model(
            ((("extended",), MessagesGetHistoryExtendedResponse),),
            default=MessagesGetHistoryResponse,
            params=params,
        )
        return model(**response).response

    async def get_history_attachments(
        self,
        attachment_position: int | None = None,
        attachment_types: list[
            typing.Literal[
                "app_action_games",
                "app_action_mini_apps",
                "audio",
                "audio_message",
                "clip",
                "doc",
                "graffiti",
                "link",
                "market",
                "photo",
                "share",
                "video",
                "wall",
            ]
        ]
        | None = None,
        cmid: int | None = None,
        count: int | None = None,
        extended: bool | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        max_forwards_level: int | None = None,
        media_type: str | None = None,
        message_video: bool | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        photo_sizes: bool | None = None,
        preserve_order: bool | None = None,
        start_from: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetHistoryAttachmentsResponseModel:
        """Method `messages.getHistoryAttachments()`

        :param attachment_position:
        :param attachment_types:
        :param cmid:
        :param count: Number of objects to return.
        :param extended:
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param group_id: Group ID (for group messages with group access token)
        :param max_forwards_level:
        :param media_type: Type of media files to return: *'photo',, *'video',, *'audio',, *'doc',, *'link'.,*'market'.,*'wall'.,*'share'
        :param message_video:
        :param offset:
        :param peer_id: Peer ID. ", For group chat: '2000000000 + chat ID' , , For community: '-community ID'"
        :param photo_sizes: '1' - to return photo sizes in a
        :param preserve_order:
        :param start_from: Message ID to start return results from.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getHistoryAttachments", params)
        model = MessagesGetHistoryAttachmentsResponse
        return model(**response).response

    @typing.overload
    async def get_important_messages(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        preview_length: int | None = None,
        start_message_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetImportantMessagesExtendedResponseModel: ...

    @typing.overload
    async def get_important_messages(
        self,
        extended: typing.Literal[False] | None = None,
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        preview_length: int | None = None,
        start_message_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetImportantMessagesResponseModel: ...

    async def get_important_messages(
        self,
        extended: bool | None = None,
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        preview_length: int | None = None,
        start_message_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetImportantMessagesExtendedResponseModel | MessagesGetImportantMessagesResponseModel:
        """Method `messages.getImportantMessages()`

        :param extended: Return extended properties
        :param count: Amount of needed important messages.
        :param fields: Actors fields to return.
        :param group_id: Group ID (for group messages with group access token)
        :param offset:
        :param preview_length: Maximum length of messages body.
        :param start_message_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getImportantMessages", params)
        model = self.get_model(
            ((("extended",), MessagesGetImportantMessagesExtendedResponse),),
            default=MessagesGetImportantMessagesResponse,
            params=params,
        )
        return model(**response).response

    async def get_intent_users(
        self,
        intent: str,
        count: int | None = None,
        extended: bool | None = None,
        fields: list[str] | None = None,
        name_case: str | None = None,
        offset: int | None = None,
        subscribe_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetIntentUsersResponseModel:
        """Method `messages.getIntentUsers()`

        :param intent:
        :param count:
        :param extended:
        :param fields:
        :param name_case:
        :param offset:
        :param subscribe_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getIntentUsers", params)
        model = MessagesGetIntentUsersResponse
        return model(**response).response

    async def get_invite_link(
        self,
        peer_id: int,
        group_id: int | None = None,
        reset: bool | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetInviteLinkResponseModel:
        """Method `messages.getInviteLink()`

        :param peer_id: Destination ID.
        :param group_id: Group ID
        :param reset: 1 - to generate new link (revoke previous), 0 - to return previous link.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getInviteLink", params)
        model = MessagesGetInviteLinkResponse
        return model(**response).response

    async def get_last_activity(
        self,
        user_id: int,
        **kwargs: typing.Any,
    ) -> "MessagesLastActivity":
        """Method `messages.getLastActivity()`

        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getLastActivity", params)
        model = MessagesGetLastActivityResponse
        return model(**response).response

    async def get_long_poll_history(
        self,
        credentials: bool | None = None,
        events_limit: int | None = None,
        extended: bool | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        last_n: int | None = None,
        lp_version: int | None = None,
        max_msg_id: int | None = None,
        msgs_limit: int | None = None,
        onlines: bool | None = None,
        preview_length: int | None = None,
        pts: int | None = None,
        ts: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetLongPollHistoryResponseModel:
        """Method `messages.getLongPollHistory()`

        :param credentials:
        :param events_limit: Maximum number of events to return.
        :param extended:
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param group_id: Group ID (for group messages with user access token)
        :param last_n:
        :param lp_version:
        :param max_msg_id: Maximum ID of the message among existing ones in the local copy. Both messages received with API methods (for example, , ), and data received from a Long Poll server (events with code 4) are taken into account.
        :param msgs_limit: Maximum number of messages to return.
        :param onlines: '1' - to return history with online users only.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param pts: Last value of 'pts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param ts: Last value of the 'ts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getLongPollHistory", params)
        model = MessagesGetLongPollHistoryResponse
        return model(**response).response

    async def get_long_poll_server(
        self,
        group_id: int | None = None,
        lp_version: int | None = None,
        need_pts: bool | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesLongpollParams":
        """Method `messages.getLongPollServer()`

        :param group_id: Group ID (for group messages with user access token)
        :param lp_version: Long poll version
        :param need_pts: '1' - to return the 'pts' field, needed for the [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getLongPollServer", params)
        model = MessagesGetLongPollServerResponse
        return model(**response).response

    async def get_messages_reactions(
        self,
        cmids: list[int],
        peer_id: int,
        **kwargs: typing.Any,
    ) -> MessagesGetMessagesReactionsResponseModel:
        """Method `messages.getMessagesReactions()`

        :param cmids:
        :param peer_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getMessagesReactions", params)
        model = MessagesGetMessagesReactionsResponse
        return model(**response).response

    async def get_reacted_peers(
        self,
        cmid: int,
        peer_id: int,
        reaction_id: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetReactedPeersResponseModel:
        """Method `messages.getReactedPeers()`

        :param cmid:
        :param peer_id:
        :param reaction_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getReactedPeers", params)
        model = MessagesGetReactedPeersResponse
        return model(**response).response

    async def get_reactions_assets(
        self,
        client_version: int | None = None,
        **kwargs: typing.Any,
    ) -> MessagesGetReactionsAssetsResponseModel:
        """Method `messages.getReactionsAssets()`

        :param client_version:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.getReactionsAssets", params)
        model = MessagesGetReactionsAssetsResponse
        return model(**response).response

    async def is_messages_from_group_allowed(
        self,
        group_id: int,
        user_id: int,
        **kwargs: typing.Any,
    ) -> MessagesIsMessagesFromGroupAllowedResponseModel:
        """Method `messages.isMessagesFromGroupAllowed()`

        :param group_id: Group ID.
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.isMessagesFromGroupAllowed", params)
        model = MessagesIsMessagesFromGroupAllowedResponse
        return model(**response).response

    async def join_chat_by_invite_link(
        self,
        link: str,
        **kwargs: typing.Any,
    ) -> MessagesJoinChatByInviteLinkResponseModel:
        """Method `messages.joinChatByInviteLink()`

        :param link: Invitation link.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.joinChatByInviteLink", params)
        model = MessagesJoinChatByInviteLinkResponse
        return model(**response).response

    async def mark_as_answered_conversation(
        self,
        peer_id: int,
        answered: bool | None = None,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.markAsAnsweredConversation()`

        :param peer_id: ID of conversation to mark as important.
        :param answered: '1' - to mark as answered, '0' - to remove the mark
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.markAsAnsweredConversation", params)
        model = BaseOkResponse
        return model(**response).response

    async def mark_as_important(
        self,
        important: int | None = None,
        message_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> list[int]:
        """Method `messages.markAsImportant()`

        :param important: '1' - to add a star (mark as important), '0' - to remove the star
        :param message_ids: IDs of messages to mark as important.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.markAsImportant", params)
        model = MessagesMarkAsImportantDeprecatedResponse
        return model(**response).response

    async def mark_as_important_conversation(
        self,
        peer_id: int,
        group_id: int | None = None,
        important: bool | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.markAsImportantConversation()`

        :param peer_id: ID of conversation to mark as important.
        :param group_id: Group ID (for group messages with group access token)
        :param important: '1' - to add a star (mark as important), '0' - to remove the star
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.markAsImportantConversation", params)
        model = BaseOkResponse
        return model(**response).response

    async def mark_as_read(
        self,
        group_id: int | None = None,
        mark_conversation_as_read: bool | None = None,
        message_ids: list[int] | None = None,
        peer_id: int | None = None,
        start_message_id: int | None = None,
        up_to_cmid: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.markAsRead()`

        :param group_id: Group ID (for group messages with user access token)
        :param mark_conversation_as_read:
        :param message_ids: IDs of messages to mark as read.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param start_message_id: Message ID to start from.
        :param up_to_cmid:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.markAsRead", params)
        model = BaseOkResponse
        return model(**response).response

    async def mark_reactions_as_read(
        self,
        peer_id: int,
        cmids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `messages.markReactionsAsRead()`

        :param peer_id:
        :param cmids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.markReactionsAsRead", params)
        model = BaseBoolResponse
        return model(**response).response

    async def mute_chat_mentions(
        self,
        mention_status: str,
        peer_id: int,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.muteChatMentions()`

        :param mention_status:
        :param peer_id: Chat id
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.muteChatMentions", params)
        model = BaseOkResponse
        return model(**response).response

    async def pin(
        self,
        peer_id: int,
        cmid: int | None = None,
        message_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "MessagesPinnedMessage":
        """Method `messages.pin()`

        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param cmid: Conversation message ID
        :param message_id: Message ID
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.pin", params)
        model = MessagesPinResponse
        return model(**response).response

    async def remove_chat_user(
        self,
        chat_id: int,
        member_id: int | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.removeChatUser()`

        :param chat_id: Chat ID.
        :param member_id:
        :param user_id: ID of the user to be removed from the chat.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.removeChatUser", params)
        model = BaseOkResponse
        return model(**response).response

    async def restore(
        self,
        cmid: int | None = None,
        group_id: int | None = None,
        message_id: int | None = None,
        peer_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.restore()`

        :param cmid:
        :param group_id: Group ID (for group messages with user access token)
        :param message_id: ID of a previously-deleted message to restore.
        :param peer_id: Destination ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.restore", params)
        model = BaseOkResponse
        return model(**response).response

    @typing.overload
    async def search(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        date: int | None = None,
        fields: list[str] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        preview_length: int | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesSearchExtendedResponseModel: ...

    @typing.overload
    async def search(
        self,
        extended: typing.Literal[False] | None = None,
        count: int | None = None,
        date: int | None = None,
        fields: list[str] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        preview_length: int | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesSearchResponseModel: ...

    async def search(
        self,
        extended: bool | None = None,
        count: int | None = None,
        date: int | None = None,
        fields: list[str] | None = None,
        group_id: int | None = None,
        offset: int | None = None,
        peer_id: int | None = None,
        preview_length: int | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesSearchExtendedResponseModel | MessagesSearchResponseModel:
        """Method `messages.search()`

        :param extended:
        :param count: Number of messages to return.
        :param date: Date to search message before in Unixtime.
        :param fields:
        :param group_id: Group ID (for group messages with group access token)
        :param offset: Offset needed to return a specific subset of messages.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param q: Search query string.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.search", params)
        model = self.get_model(
            ((("extended",), MessagesSearchExtendedResponse),),
            default=MessagesSearchResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def search_conversations(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesSearchConversationsExtendedResponseModel: ...

    @typing.overload
    async def search_conversations(
        self,
        extended: typing.Literal[False] | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesSearchConversationsResponseModel: ...

    async def search_conversations(
        self,
        extended: bool | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        group_id: int | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> MessagesSearchConversationsExtendedResponseModel | MessagesSearchConversationsResponseModel:
        """Method `messages.searchConversations()`

        :param extended: '1' - return extra information about users and communities
        :param count: Maximum number of results.
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with user access token)
        :param q: Search query string.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.searchConversations", params)
        model = self.get_model(
            ((("extended",), MessagesSearchConversationsExtendedResponse),),
            default=MessagesSearchConversationsResponse,
            params=params,
        )
        return model(**response).response

    async def send(
        self,
        attachment: str | None = None,
        chat_id: int | None = None,
        content_source: str | None = None,
        disable_mentions: bool | None = None,
        domain: str | None = None,
        dont_parse_links: bool | None = None,
        forward: str | None = None,
        forward_messages: list[int] | None = None,
        group_id: int | None = None,
        intent: str | None = None,
        keyboard: str | None = None,
        lat: float | None = None,
        long: float | None = None,
        message: str | None = None,
        payload: str | None = None,
        peer_id: int | None = None,
        peer_ids: list[int] | None = None,
        random_id: int | None = None,
        reply_to: int | None = None,
        sticker_id: int | None = None,
        subscribe_id: int | None = None,
        template: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> dict[str, typing.Any]:
        """Method `messages.send()`

        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, 'wall' - wall post, '<owner_id>' - ID of the media attachment owner. '<media_id>' - media attachment ID. Example: "photo100172_166443618"
        :param chat_id: ID of conversation the message will relate to.
        :param content_source: JSON describing the content source in the message
        :param disable_mentions:
        :param domain: User's short address (for example, 'illarionov').
        :param dont_parse_links:
        :param forward: JSON describing the forwarded message or reply
        :param forward_messages: ID of forwarded messages, separated with a comma. Listed messages of the sender will be shown in the message body at the recipient's. Example: "123,431,544"
        :param group_id: Group ID (for group messages with group access token)
        :param intent:
        :param keyboard:
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param payload:
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param peer_ids: IDs of message recipients. (See peer_id)
        :param random_id: Unique identifier to avoid resending the message.
        :param reply_to:
        :param sticker_id: Sticker id.
        :param subscribe_id:
        :param template:
        :param user_id: User ID (by default - current user).
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.send", params)
        model = DictResponse
        return model(**response).response

    async def send_message_event_answer(
        self,
        event_id: str,
        peer_id: int,
        user_id: int,
        event_data: str | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.sendMessageEventAnswer()`

        :param event_id:
        :param peer_id:
        :param user_id:
        :param event_data:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.sendMessageEventAnswer", params)
        model = BaseOkResponse
        return model(**response).response

    async def send_reaction(
        self,
        cmid: int,
        peer_id: int,
        reaction_id: int,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `messages.sendReaction()`

        :param cmid:
        :param peer_id:
        :param reaction_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.sendReaction", params)
        model = BaseBoolResponse
        return model(**response).response

    async def set_activity(
        self,
        group_id: int | None = None,
        peer_id: int | None = None,
        type: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.setActivity()`

        :param group_id: Group ID (for group messages with group access token)
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param type: 'typing' - user has started to type.
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.setActivity", params)
        model = BaseOkResponse
        return model(**response).response

    async def set_chat_photo(
        self,
        file: str,
        **kwargs: typing.Any,
    ) -> MessagesSetChatPhotoResponseModel:
        """Method `messages.setChatPhoto()`

        :param file: Upload URL from the 'response' field returned by the [vk.com/dev/photos.getChatUploadServer|photos.getChatUploadServer] method upon successfully uploading an image.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.setChatPhoto", params)
        model = MessagesSetChatPhotoResponse
        return model(**response).response

    async def unpin(
        self,
        peer_id: int,
        group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `messages.unpin()`

        :param peer_id:
        :param group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("messages.unpin", params)
        model = BaseOkResponse
        return model(**response).response


__all__ = ("MessagesCategory",)



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import (
    AccountAccountCounters,
    AccountInfo,
    AccountNameRequest,
    AccountOffer,
    AccountPushSettings,
    AccountUserSettings,
    GroupsGroup,
    UsersUserFull,
)
from vkbottle_types.responses.base_response import BaseResponse


class AccountChangePasswordResponseModel(BaseModel):
    token: str = Field()
    secret: str | None = Field(
        default=None,
    )


class AccountChangePasswordResponse(BaseResponse):
    response: "AccountChangePasswordResponseModel" = Field()


class AccountGetActiveOffersResponseModel(BaseModel):
    count: int = Field()
    items: list["AccountOffer"] = Field()


class AccountGetActiveOffersResponse(BaseResponse):
    response: "AccountGetActiveOffersResponseModel" = Field()


class AccountGetAppPermissionsResponse(BaseResponse):
    response: int = Field()


class AccountGetBannedResponseModel(BaseModel):
    count: int = Field()
    items: list[int] = Field()
    profiles: list["UsersUserFull"] | None = Field(
        default=None,
    )
    groups: list["GroupsGroup"] | None = Field(
        default=None,
    )


class AccountGetBannedResponse(BaseResponse):
    response: "AccountGetBannedResponseModel" = Field()


class AccountGetCountersResponse(BaseResponse):
    response: "AccountAccountCounters" = Field()


class AccountGetInfoResponse(BaseResponse):
    response: "AccountInfo" = Field()


class AccountGetProfileInfoResponse(BaseResponse):
    response: "AccountUserSettings" = Field()


class AccountGetPushSettingsResponse(BaseResponse):
    response: "AccountPushSettings" = Field()


class AccountSaveProfileInfoResponseModel(BaseModel):
    changed: bool = Field()
    name_request: "AccountNameRequest | None" = Field(
        default=None,
    )


class AccountSaveProfileInfoResponse(BaseResponse):
    response: "AccountSaveProfileInfoResponseModel" = Field()



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import BaseUserGroupFields
from vkbottle_types.responses.base import (
    BaseBoolResponse,
    BaseOkResponse,
    BaseOkResponseModel,
)
from vkbottle_types.responses.wall import *  # noqa: F401,F403  # type: ignore


class WallCategory(BaseCategory):
    async def check_copyright_link(
        self,
        link: str,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `wall.checkCopyrightLink()`

        :param link:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.checkCopyrightLink", params)
        model = BaseBoolResponse
        return model(**response).response

    async def close_comments(
        self,
        owner_id: int,
        post_id: int,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `wall.closeComments()`

        :param owner_id:
        :param post_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.closeComments", params)
        model = BaseBoolResponse
        return model(**response).response

    async def create_comment(
        self,
        post_id: int,
        attachments: list[str] | None = None,
        from_group: int | None = None,
        guid: str | None = None,
        message: str | None = None,
        owner_id: int | None = None,
        reply_to_comment: int | None = None,
        sticker_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WallCreateCommentResponseModel:
        """Method `wall.createComment()`

        :param post_id: Post ID.
        :param attachments: (Required if 'message' is not set.) List of media objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' - Type of media ojbect: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, '<owner_id>' - ID of the media owner. '<media_id>' - Media ID. For example: "photo100172_166443618,photo66748_265827614"
        :param from_group: Group ID.
        :param guid: Unique identifier to avoid repeated comments.
        :param message: (Required if 'attachments' is not set.) Text of the comment.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param reply_to_comment: ID of comment to reply.
        :param sticker_id: Sticker ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.createComment", params)
        model = WallCreateCommentResponse
        return model(**response).response

    async def delete(
        self,
        owner_id: int | None = None,
        post_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.delete()`

        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: ID of the post to be deleted.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.delete", params)
        model = BaseOkResponse
        return model(**response).response

    async def delete_comment(
        self,
        comment_id: int,
        owner_id: int | None = None,
        post_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.deleteComment()`

        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.deleteComment", params)
        model = BaseOkResponse
        return model(**response).response

    async def edit(
        self,
        post_id: int,
        attachments: list[str] | None = None,
        close_comments: bool | None = None,
        copyright: str | None = None,
        donut_paid_duration: int | None = None,
        friends_only: bool | None = None,
        lat: float | None = None,
        long: float | None = None,
        mark_as_ads: bool | None = None,
        message: str | None = None,
        owner_id: int | None = None,
        place_id: int | None = None,
        poster_bkg_access_hash: str | None = None,
        poster_bkg_id: int | None = None,
        poster_bkg_owner_id: int | None = None,
        publish_date: int | None = None,
        services: str | None = None,
        signed: bool | None = None,
        topic_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WallEditResponseModel:
        """Method `wall.edit()`

        :param post_id:
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, '<owner_id>' - ID of the media application owner. '<media_id>' - Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error is thrown."
        :param close_comments:
        :param copyright:
        :param donut_paid_duration:
        :param friends_only:
        :param lat:
        :param long:
        :param mark_as_ads:
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param place_id:
        :param poster_bkg_access_hash:
        :param poster_bkg_id:
        :param poster_bkg_owner_id:
        :param publish_date:
        :param services:
        :param signed:
        :param topic_id: Topic ID. Allowed values can be obtained from newsfeed.getPostTopics method
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.edit", params)
        model = WallEditResponse
        return model(**response).response

    async def edit_ads_stealth(
        self,
        post_id: int,
        attachments: list[str] | None = None,
        lat: float | None = None,
        link_button: str | None = None,
        link_image: str | None = None,
        link_title: str | None = None,
        link_video: str | None = None,
        long: float | None = None,
        message: str | None = None,
        owner_id: int | None = None,
        place_id: int | None = None,
        signed: bool | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.editAdsStealth()`

        :param post_id: Post ID. Used for publishing of scheduled and suggested posts.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, 'page' - wiki-page, 'note' - note, 'poll' - poll, 'album' - photo album, '<owner_id>' - ID of the media application owner. '<media_id>' - Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param link_button: Link button ID
        :param link_image: Link image url
        :param link_title: Link title
        :param link_video: Link video ID in format "<owner_id>_<media_id>"
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param place_id: ID of the location where the user was tagged.
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' - post will be signed with the name of the posting user, '0' - post will not be signed (default)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.editAdsStealth", params)
        model = BaseOkResponse
        return model(**response).response

    async def edit_comment(
        self,
        comment_id: int,
        attachments: list[str] | None = None,
        message: str | None = None,
        owner_id: int | None = None,
        post_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.editComment()`

        :param comment_id: Comment ID.
        :param attachments: List of objects attached to the comment, in the following format: , "<owner_id>_<media_id>,<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, '<owner_id>' - ID of the media attachment owner. '<media_id>' - Media attachment ID. For example: "photo100172_166443618,photo66748_265827614"
        :param message: New comment text.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.editComment", params)
        model = BaseOkResponse
        return model(**response).response

    @typing.overload
    async def get(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        domain: int | str | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        filter: str | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetExtendedResponseModel: ...

    @typing.overload
    async def get(
        self,
        extended: typing.Literal[False] | None = None,
        count: int | None = None,
        domain: int | str | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        filter: str | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetResponseModel: ...

    async def get(
        self,
        extended: bool | None = None,
        count: int | None = None,
        domain: int | str | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        filter: str | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetResponseModel | WallGetExtendedResponseModel:
        """Method `wall.get()`

        :param extended: '1' - to return 'wall', 'profiles', and 'groups' fields, '0' - to return no additional fields (default)
        :param count: Number of posts to return (maximum 100).
        :param domain: User or community short address.
        :param fields:
        :param filter:
        :param offset: Offset needed to return a specific subset of posts.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.get", params)
        model = self.get_model(
            ((("extended",), WallGetExtendedResponse),),
            default=WallGetResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_by_id(
        self,
        posts: list[str],
        extended: typing.Literal[True],
        copy_history_depth: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        **kwargs: typing.Any,
    ) -> WallGetByIdExtendedResponseModel: ...

    @typing.overload
    async def get_by_id(
        self,
        posts: list[str],
        extended: typing.Literal[False] | None = None,
        copy_history_depth: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        **kwargs: typing.Any,
    ) -> WallGetByIdResponseModel: ...

    async def get_by_id(
        self,
        posts: list[str],
        extended: bool | None = None,
        copy_history_depth: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        **kwargs: typing.Any,
    ) -> WallGetByIdExtendedResponseModel | WallGetByIdResponseModel:
        """Method `wall.getById()`

        :param posts: User or community IDs and post IDs, separated by underscores. Use a negative value to designate a community ID. Example: "93388_21539,93388_20904,2943_4276,-1_1"
        :param extended: '1' - to return user and community objects needed to display posts, '0' - no additional fields are returned (default)
        :param copy_history_depth: Sets the number of parent elements to include in the array 'copy_history' that is returned if the post is a repost from another wall.
        :param fields:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.getById", params)
        model = self.get_model(
            ((("extended",), WallGetByIdExtendedResponse),),
            default=WallGetByIdResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_comment(
        self,
        comment_id: int,
        extended: typing.Literal[True],
        fields: list[BaseUserGroupFields] | None = None,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetCommentExtendedResponseModel: ...

    @typing.overload
    async def get_comment(
        self,
        comment_id: int,
        extended: typing.Literal[False] | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetCommentResponseModel: ...

    async def get_comment(
        self,
        comment_id: int,
        extended: bool | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetCommentExtendedResponseModel | WallGetCommentResponseModel:
        """Method `wall.getComment()`

        :param comment_id: Comment ID.
        :param extended:
        :param fields:
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.getComment", params)
        model = self.get_model(
            ((("extended",), WallGetCommentExtendedResponse),),
            default=WallGetCommentResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_comments(
        self,
        extended: typing.Literal[True],
        comment_id: int | None = None,
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        need_likes: bool | None = None,
        offset: int | None = None,
        owner_id: int | None = None,
        post_id: int | None = None,
        preview_length: int | None = None,
        sort: str | None = None,
        start_comment_id: int | None = None,
        thread_items_count: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetCommentsExtendedResponseModel: ...

    @typing.overload
    async def get_comments(
        self,
        extended: typing.Literal[False] | None = None,
        comment_id: int | None = None,
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        need_likes: bool | None = None,
        offset: int | None = None,
        owner_id: int | None = None,
        post_id: int | None = None,
        preview_length: int | None = None,
        sort: str | None = None,
        start_comment_id: int | None = None,
        thread_items_count: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetCommentsResponseModel: ...

    async def get_comments(
        self,
        extended: bool | None = None,
        comment_id: int | None = None,
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        need_likes: bool | None = None,
        offset: int | None = None,
        owner_id: int | None = None,
        post_id: int | None = None,
        preview_length: int | None = None,
        sort: str | None = None,
        start_comment_id: int | None = None,
        thread_items_count: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetCommentsResponseModel | WallGetCommentsExtendedResponseModel:
        """Method `wall.getComments()`

        :param extended:
        :param comment_id: Comment ID.
        :param count: Number of comments to return (maximum 100).
        :param fields:
        :param need_likes: '1' - to return the 'likes' field, '0' - not to return the 'likes' field (default)
        :param offset: Offset needed to return a specific subset of comments.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        :param preview_length: Number of characters at which to truncate comments when previewed. By default, '90'. Specify '0' if you do not want to truncate comments.
        :param sort: Sort order: 'asc' - chronological, 'desc' - reverse chronological
        :param start_comment_id:
        :param thread_items_count: Count items in threads.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.getComments", params)
        model = self.get_model(
            ((("extended",), WallGetCommentsExtendedResponse),),
            default=WallGetCommentsResponse,
            params=params,
        )
        return model(**response).response

    async def get_reposts(
        self,
        count: int | None = None,
        offset: int | None = None,
        owner_id: int | None = None,
        post_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WallGetRepostsResponseModel:
        """Method `wall.getReposts()`

        :param count: Number of reposts to return.
        :param offset: Offset needed to return a specific subset of reposts.
        :param owner_id: User ID or community ID. By default, current user ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.getReposts", params)
        model = WallGetRepostsResponse
        return model(**response).response

    async def open_comments(
        self,
        owner_id: int,
        post_id: int,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `wall.openComments()`

        :param owner_id:
        :param post_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.openComments", params)
        model = BaseBoolResponse
        return model(**response).response

    async def parse_attached_link(
        self,
        links: str,
        extended: bool | None = None,
        fields: list[str] | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> WallParseAttachedLinkResponseModel:
        """Method `wall.parseAttachedLink()`

        :param links:
        :param extended:
        :param fields:
        :param name_case:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.parseAttachedLink", params)
        model = WallParseAttachedLinkResponse
        return model(**response).response

    async def pin(
        self,
        post_id: int,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.pin()`

        :param post_id: Post ID.
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.pin", params)
        model = BaseOkResponse
        return model(**response).response

    async def post(
        self,
        attachments: list[str] | None = None,
        close_comments: bool | None = None,
        copyright: str | None = None,
        donut_paid_duration: int | None = None,
        friends_only: bool | None = None,
        from_group: bool | None = None,
        guid: str | None = None,
        lat: float | None = None,
        link_photo_id: str | None = None,
        link_title: str | None = None,
        long: float | None = None,
        mark_as_ads: bool | None = None,
        message: str | None = None,
        mute_notifications: bool | None = None,
        owner_id: int | None = None,
        place_id: int | None = None,
        post_id: int | None = None,
        publish_date: int | None = None,
        services: str | None = None,
        signed: bool | None = None,
        **kwargs: typing.Any,
    ) -> WallPostResponseModel:
        """Method `wall.post()`

        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, 'page' - wiki-page, 'note' - note, 'poll' - poll, 'album' - photo album, '<owner_id>' - ID of the media application owner. '<media_id>' - Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param close_comments:
        :param copyright:
        :param donut_paid_duration:
        :param friends_only: '1' - post will be available to friends only, '0' - post will be available to all users (default)
        :param from_group: For a community: '1' - post will be published by the community, '0' - post will be published by the user (default)
        :param guid:
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param link_photo_id:
        :param link_title:
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param mark_as_ads:
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param mute_notifications:
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param place_id: ID of the location where the user was tagged.
        :param post_id: Post ID. Used for publishing of scheduled and suggested posts.
        :param publish_date: Publication date (in Unix time). If used, posting will be delayed until the set time.
        :param services: List of services or websites the update will be exported to, if the user has so requested. Sample values: 'twitter', 'facebook'.
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' - post will be signed with the name of the posting user, '0' - post will not be signed (default)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.post", params)
        model = WallPostResponse
        return model(**response).response

    async def post_ads_stealth(
        self,
        owner_id: int,
        attachments: list[str] | None = None,
        guid: str | None = None,
        lat: float | None = None,
        link_button: str | None = None,
        link_image: str | None = None,
        link_title: str | None = None,
        link_video: str | None = None,
        long: float | None = None,
        message: str | None = None,
        place_id: int | None = None,
        signed: bool | None = None,
        **kwargs: typing.Any,
    ) -> WallPostAdsStealthResponseModel:
        """Method `wall.postAdsStealth()`

        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' - Type of media attachment: 'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document, 'page' - wiki-page, 'note' - note, 'poll' - poll, 'album' - photo album, '<owner_id>' - ID of the media application owner. '<media_id>' - Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param guid: Unique identifier to avoid duplication the same post.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param link_button: Link button ID
        :param link_image: Link image url
        :param link_title: Link title
        :param link_video: Link video ID in format "<owner_id>_<media_id>"
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param place_id: ID of the location where the user was tagged.
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' - post will be signed with the name of the posting user, '0' - post will not be signed (default)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.postAdsStealth", params)
        model = WallPostAdsStealthResponse
        return model(**response).response

    async def report_comment(
        self,
        comment_id: int,
        owner_id: int,
        reason: int,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.reportComment()`

        :param comment_id: Comment ID.
        :param owner_id: ID of the user or community that owns the wall.
        :param reason: Reason for the complaint: '0' - spam, '1' - child pornography, '2' - extremism, '3' - violence, '4' - drug propaganda, '5' - adult material, '6' - insult, abuse
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.reportComment", params)
        model = BaseOkResponse
        return model(**response).response

    async def report_post(
        self,
        owner_id: int,
        post_id: int,
        reason: int,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.reportPost()`

        :param owner_id: ID of the user or community that owns the wall.
        :param post_id: Post ID.
        :param reason: Reason for the complaint: '0' - spam, '1' - child pornography, '2' - extremism, '3' - violence, '4' - drug propaganda, '5' - adult material, '6' - insult, abuse
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.reportPost", params)
        model = BaseOkResponse
        return model(**response).response

    async def repost(
        self,
        object: str,
        group_id: int | None = None,
        mark_as_ads: bool | None = None,
        message: str | None = None,
        mute_notifications: bool | None = None,
        **kwargs: typing.Any,
    ) -> WallRepostResponseModel:
        """Method `wall.repost()`

        :param object: ID of the object to be reposted on the wall. Example: "wall66748_3675"
        :param group_id: Target community ID when reposting to a community.
        :param mark_as_ads:
        :param message: Comment to be added along with the reposted object.
        :param mute_notifications:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.repost", params)
        model = WallRepostResponse
        return model(**response).response

    async def restore(
        self,
        owner_id: int | None = None,
        post_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.restore()`

        :param owner_id: User ID or community ID from whose wall the post was deleted. Use a negative value to designate a community ID.
        :param post_id: ID of the post to be restored.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.restore", params)
        model = BaseOkResponse
        return model(**response).response

    async def restore_comment(
        self,
        comment_id: int,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.restoreComment()`

        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.restoreComment", params)
        model = BaseOkResponse
        return model(**response).response

    @typing.overload
    async def search(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        domain: int | str | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        offset: int | None = None,
        owners_only: bool | None = None,
        query: str | None = None,
        **kwargs: typing.Any,
    ) -> WallSearchExtendedResponseModel: ...

    @typing.overload
    async def search(
        self,
        extended: typing.Literal[False] | None = None,
        count: int | None = None,
        domain: int | str | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        offset: int | None = None,
        owners_only: bool | None = None,
        query: str | None = None,
        **kwargs: typing.Any,
    ) -> WallSearchResponseModel: ...

    async def search(
        self,
        extended: bool | None = None,
        count: int | None = None,
        domain: int | str | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        offset: int | None = None,
        owners_only: bool | None = None,
        query: str | None = None,
        **kwargs: typing.Any,
    ) -> WallSearchResponseModel | WallSearchExtendedResponseModel:
        """Method `wall.search()`

        :param extended: show extended post info.
        :param count: count of posts to return.
        :param domain: user or community screen name.
        :param fields:
        :param offset: Offset needed to return a specific subset of posts.
        :param owners_only: '1' - returns only page owner's posts.
        :param query: search query string.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.search", params)
        model = self.get_model(
            ((("extended",), WallSearchExtendedResponse),),
            default=WallSearchResponse,
            params=params,
        )
        return model(**response).response

    async def unpin(
        self,
        post_id: int,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `wall.unpin()`

        :param post_id: Post ID.
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("wall.unpin", params)
        model = BaseOkResponse
        return model(**response).response


__all__ = ("WallCategory",)



import dataclasses
import typing
from collections import OrderedDict

import dataclass_factory

from .parse_types import (
    PRIMITIVE_TYPES,
    UNIX_TIMESTAMP_DESCRIPTION_TEXT,
    Ready,
    get_complex_type,
    get_responses,
    get_type,
    transform_ref,
)
from .utility import camelcase, snake_case


@dataclasses.dataclass
class WithRef:
    ref: str | None = None
    type: str | None = None
    definition: "Definition | None" = None
    properties: dict = dataclasses.field(default_factory=lambda: {})


@dataclasses.dataclass
class MethodParameter:
    name: str
    type: str | list[str] | dict | Ready

    required: bool = False
    description: str | None = None
    default: str | int | bool | None = None

    format: str | None = None
    items: dict = dataclasses.field(default_factory=lambda: {})

    def get_type(self) -> str:
        return get_type(self.type, self.items)

    def get_default(self, default: typing.Any | None = None, tp: str | None = None) -> str:
        default_val = default or self.default
        tp = tp or self.type  # type: ignore

        if not self.required:
            return "None"

        if default is None and tp == "array":
            return "None"

        if isinstance(tp, list):
            result = None
            for t in tp:
                res = self.get_default(default_val, t)
                if res != "None":
                    result = res
                    break
            return "None" if result is None else result

        if isinstance(default_val, int) and tp in ("boolean", "bool"):
            return repr(bool(default_val))
        if isinstance(default_val, int) and tp in ("integer", "int"):
            return repr(default_val)
        if isinstance(default_val, (int, float)) and tp in ("number", "float"):
            return repr(float(default_val))
        return str(default_val)

    @property
    def has_default(self) -> bool:
        return not self.required or self.default is not None

    @classmethod
    def new_dependant(cls, orig: "MethodParameter") -> "MethodParameter":
        dependant = MethodParameter(
            orig.name,
            orig.type,
            orig.required,
            orig.description,
            orig.default,
            orig.format,
            orig.items,
        )
        if dependant.type == "boolean":
            dependant.type = Ready("typing.Literal[True]")
            dependant.required = True
            dependant.default = None
            return dependant

        if isinstance(dependant.type, Ready):
            if dependant.type.value == "typing.Literal[True]":
                dependant.type = Ready("typing.Literal[False]")
                dependant.default = None
                dependant.required = False
                return dependant
            if dependant.type.value == "typing.Literal[False]":
                orig.type = "boolean"
                orig.required = False
                orig.default = None
                return orig

        dependant.required = not dependant.required
        return dependant


@dataclasses.dataclass
class Method:
    name: str
    access_token_type: list[str]
    parameters: list[MethodParameter] = dataclasses.field(
        default_factory=lambda: [],
    )
    responses: dict[str, WithRef] = dataclasses.field(default_factory=lambda: {})
    errors: list[WithRef] = dataclasses.field(default_factory=lambda: [])
    timeout: int | None = dataclasses.field(default=None)
    dependant_parameters: list[str] = dataclasses.field(default_factory=lambda: [])

    def __post_init__(self) -> None:
        self.dependant_parameters = self.dependant_parameters or self.get_dependant_params()

    def get_dependant_params(self) -> list[str]:
        dependant_params: set[str] = set()

        for name in self.responses.copy():
            if name == "response":
                continue
            original_name = name.removesuffix("Response").removesuffix("_")
            dep_params = set(map(snake_case, original_name.split("_")))
            if not any(self.param_exists(param) for param in dep_params):
                continue
            dependant_params |= dep_params

        return list(dependant_params)

    def param_exists(self, param_name: str) -> bool:
        return any(param.name == param_name for param in self.parameters)

    def get_dependants(self) -> list["Method"]:
        if len(self.responses) > 1:
            dependant_responses = typing.cast(dict, self.responses.copy())
            dependant_responses.pop("response", None)
            dependant_responses = OrderedDict[str, dict[str, str]](
                **dependant_responses,
                response=self.responses.get("response"),  # type: ignore
            )
            dependants = []
            method_params: dict[str, MethodParameter] = {}
            dependant_params: set[str] = set(self.dependant_parameters)

            for name, response in dependant_responses.items():
                if response is None:
                    continue

                if name != "response":
                    original_param_names = name.removeprefix("response").removesuffix("Response").removesuffix("_").replace("Response", "")
                    dependant_params = dependant_params | set(x for x in map(snake_case, original_param_names.split("_")) if x not in PRIMITIVE_TYPES)
                    dependant_found = False
                else:
                    dependant_found = bool(self.dependant_parameters)

                for param in list(method_params.copy().values()) or self.parameters:
                    if param.name not in dependant_params:
                        method_params[param.name] = param
                    else:
                        dependant_found = True
                        new_dep = param.new_dependant(param)
                        method_params[param.name] = new_dep

                        if new_dep is param:
                            dependant_params.remove(param.name)
                        else:
                            dependant_params.add(param.name)

                if dependant_found:
                    dependants.append(
                        Method(
                            self.name,
                            self.access_token_type,
                            parameters=list(method_params.values()),
                            responses={"response": response},  # type: ignore
                            dependant_parameters=list(dependant_params),
                        )
                    )
                elif name != "response" and name in self.responses:
                    self.responses.pop(name)

            return dependants

        return []

    def get_dependants_dependant_parameters(self) -> dict[str, str]:
        dependant_parameters = {}
        response_model: str | None = (
            self.responses.get("response", {}).get("response_hint", {}).get("hint", None)  # type: ignore
        )

        for dependant in self.get_dependants():
            for dependant_parameter in dependant.dependant_parameters:
                dependant_response_model = dependant.get_response_model()
                if response_model == dependant_response_model or dependant_parameter in dependant_parameters:
                    continue
                dependant_parameters[dependant_parameter] = dependant_response_model

        return dependant_parameters

    def get_responses_hints(self) -> list[str]:
        return list(
            set(
                get_complex_type(response, response=True)
                for response in get_responses(self.responses)  # type: ignore
            ),
        )

    def get_response_hint(self) -> str:
        responses = get_responses(self.responses)  # type: ignore
        if not responses:
            return "dict[str, typing.Any]"
        if len(responses) > 1:
            responses_hints = self.get_responses_hints()
            if len(responses_hints) == 1 or not self.get_dependants():
                return responses_hints[0]
            union = " | ".join(x.strip('"').strip("'") for x in responses_hints)
            return "{}".format(f'"{union}"' if any("Literal" not in x and ("'" in x or '"' in x) for x in responses_hints) else union)
        return get_complex_type(responses[0], response=True)

    def get_response_model(self) -> str:
        responses = get_responses(self.responses)  # type: ignore
        if not responses:
            return "DictResponse"
        return get_complex_type(responses[0])  # type: ignore

    @property
    def is_single_response(self) -> bool:
        return len(self.responses) <= 1

    @property
    def sorted_parameters(self) -> list["MethodParameter"]:
        required = []
        optional = []

        for param in self.parameters:
            if not param.has_default:
                required.append(param)
            else:
                optional.append(param)

        required.sort(key=lambda param: param.name)
        optional.sort(key=lambda param: param.name)

        if not self.dependant_parameters:
            return required + optional

        dependant_required = []
        dependant_optional = []

        for param_name in self.dependant_parameters:
            for index, param in enumerate(required.copy()):
                if param.name == param_name:
                    dependant_required.append(required.pop(index))

            for index, param in enumerate(optional.copy()):
                if param.name == param_name:
                    dependant_optional.append(optional.pop(index))

        return (required + dependant_required) + (dependant_optional + optional)


def wrap_optional(func):
    def wrapper(self: "Property", **kw: typing.Any) -> str:
        response = func(self, **kw)
        if self.is_optional:
            response = (
                "{} | None".format(response)
                if not response.startswith("'") or not response.endswith("'")
                else '"{} | None"'.format(response.strip('"').strip("'"))
            )
        return response

    return wrapper


@dataclasses.dataclass
class Property:
    name: str
    type: str | list[str] | None = dataclasses.field(default=None)
    description: str | None = dataclasses.field(default=None)
    default: typing.Any | None = dataclasses.field(default=None)
    enum: list[str | int] = dataclasses.field(default_factory=lambda: [])
    enumNames: list[str] = dataclasses.field(default_factory=lambda: [])  # noqa: N815
    items: dict = dataclasses.field(default_factory=lambda: {})
    ref: str | None = dataclasses.field(default=None)
    required: bool = dataclasses.field(default=False)
    data: dict = dataclasses.field(default_factory=lambda: {})

    @wrap_optional
    def get_type(
        self,
        hint: bool = False,
        response: bool = False,
        literal_to_enum: bool = False,
        definition_name: str | None = None,
    ) -> str:
        if not self.type:
            ct = get_complex_type({"$ref": self.ref}, hint=True)

            if ct == "'BaseBoolInt'":
                return "bool"

            return ct

        t = get_type(self.type, self.items or self.data, hint=hint)
        if literal_to_enum and t.startswith("typing.Literal"):
            if not definition_name:
                raise LookupError("Definition name is required")
            propname = camelcase(definition_name) + camelcase(self.name)
            return repr(propname)

        if t == "dict" and response:
            return "dict[str, typing.Any]"
        return t

    @property
    def is_optional(self) -> bool:
        return not self.required

    def get_default_value(self, model_name: str | None = None) -> typing.Any | None:
        if self.default is None:
            return None
        if self.enum and model_name:
            for i, enum in enumerate(self.enum):
                if enum == self.default:
                    return model_name + "." + self.enumNames[i].replace(" ", "_").upper()
        if self.type in ("boolean", "BaseBoolInt"):
            return bool(self.default)
        return self.default


@dataclasses.dataclass
class Definition:
    type: str
    allOf: list[WithRef] = dataclasses.field(default_factory=lambda: [])  # noqa: N815
    properties: list[Property] = dataclasses.field(default_factory=lambda: [])
    enum: list[str | int] = dataclasses.field(default_factory=lambda: [])
    enumNames: list[str] = dataclasses.field(default_factory=lambda: [])  # noqa: N815
    sub_definitions: dict[str, "Definition"] = dataclasses.field(default_factory=lambda: {})
    items: typing.Any | None = None
    ref: str | None = None
    name: str | None = None
    description: str | None = None

    _bases: list[str] = dataclasses.field(default_factory=lambda: [])
    _default_base: str = "BaseModel"
    _default_base_response: str = "BaseResponse"

    @property
    def is_typealias(self) -> bool:
        return bool(self.type == "integer" and UNIX_TIMESTAMP_DESCRIPTION_TEXT in (self.description or "")) or (
            self.type != "object" and not any((self.properties, self.allOf, self.enum, self.sub_definitions))
        )

    @property
    def typealias_value(self) -> str | None:
        return None if not self.is_typealias else get_type(self.type, dataclasses.asdict(self), hint=True)

    def get_response_definition(self) -> "Definition | None":
        for prop in self.properties:
            if prop.name != "response":
                continue
            if prop.data.get("enum"):
                return Definition(
                    type="enum",
                    enum=prop.data["enum"],
                    enumNames=prop.data.get("enumNames", []),
                )
            if prop.data.get("type") == "object":
                properties = [factory.load({"name": pname, **p}, Property) for pname, p in prop.data.get("properties", {}).items()]
                return Definition(type="object", allOf=self.allOf, properties=properties)
        return None

    def get_properties(self) -> list[Property]:
        return sorted(self.properties, key=lambda x: x.is_optional)

    def get_enums(self, definition_name: str) -> typing.Iterator["Definition"]:
        for prop in self.get_properties():
            if prop.type == "string" and (prop.data or {}).get("enum"):
                yield Definition(
                    "enum",
                    enum=prop.data["enum"],
                    name=camelcase(definition_name) + camelcase(prop.name),
                )

    def get_typehint(self) -> str:
        assert self.type != "object"
        return get_complex_type(dataclasses.asdict(self), hint=True)

    def get_bases(self, response: bool = False) -> list[str]:
        if self._bases:
            return self._bases

        if self.allOf:
            base_sub_definitions: dict[str, list[str]] = {
                transform_ref(base.ref): list(base.definition.sub_definitions.keys()) for base in self.allOf if base.ref and base.definition
            }
            refs = [transform_ref(base.ref) for base in self.allOf if base.ref]

            for ref in refs.copy():
                if ref in base_sub_definitions:
                    for sub_def in base_sub_definitions[ref]:
                        if sub_def in refs:
                            refs.remove(ref)

            for base in self.allOf:
                if base.properties:
                    for propname, prop in base.properties.items():
                        self.properties.append(factory.load({"name": propname, **prop}, Property))
                    pass

            self._bases = refs
            return self._bases

        self._bases = [self._default_base if not response else self._default_base_response]
        return self._bases


@dataclasses.dataclass
class Objects:
    title: str
    definitions: dict[str, Definition] = dataclasses.field(default_factory=lambda: {})
    version: str | None = None
    description: str | None = None


@dataclasses.dataclass
class Category:
    name: str
    methods: list[Method] = dataclasses.field(default_factory=lambda: [])
    objects: Objects | None = None
    responses: Objects | None = None


ref_schema: dataclass_factory.Schema = dataclass_factory.Schema(name_mapping={"ref": "$ref"})
factory = dataclass_factory.Factory(
    schemas={WithRef: ref_schema, Property: ref_schema, Definition: ref_schema},
)


__all__ = (
    "Method",
    "MethodParameter",
    "Property",
    "Definition",
    "Objects",
    "Category",
    "factory",
)



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.responses.base import (
    BaseOkResponse,
    BaseOkResponseModel,
)
from vkbottle_types.responses.store import *  # noqa: F401,F403  # type: ignore


class StoreCategory(BaseCategory):
    async def add_stickers_to_favorite(
        self,
        sticker_ids: list[int],
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `store.addStickersToFavorite()`

        :param sticker_ids: Sticker IDs to be added
        """

        params = self.get_set_params(locals())
        response = await self.api.request("store.addStickersToFavorite", params)
        model = BaseOkResponse
        return model(**response).response

    async def get_favorite_stickers(
        self,
        **kwargs: typing.Any,
    ) -> StoreGetFavoriteStickersResponseModel:
        """Method `store.getFavoriteStickers()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("store.getFavoriteStickers", params)
        model = StoreGetFavoriteStickersResponse
        return model(**response).response

    async def get_products(
        self,
        extended: bool | None = None,
        filters: list[str] | None = None,
        merchant: str | None = None,
        product_ids: list[int] | None = None,
        section: str | None = None,
        type: str | None = None,
        **kwargs: typing.Any,
    ) -> StoreGetProductsResponseModel:
        """Method `store.getProducts()`

        :param extended:
        :param filters:
        :param merchant:
        :param product_ids:
        :param section:
        :param type:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("store.getProducts", params)
        model = StoreGetProductsResponse
        return model(**response).response

    async def get_stickers_keywords(
        self,
        aliases: bool | None = None,
        all_products: bool | None = None,
        need_stickers: bool | None = None,
        products_ids: list[int] | None = None,
        stickers_ids: list[int] | None = None,
        vmoji_promo: bool | None = None,
        **kwargs: typing.Any,
    ) -> StoreGetStickersKeywordsResponseModel:
        """Method `store.getStickersKeywords()`

        :param aliases:
        :param all_products:
        :param need_stickers:
        :param products_ids:
        :param stickers_ids:
        :param vmoji_promo:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("store.getStickersKeywords", params)
        model = StoreGetStickersKeywordsResponse
        return model(**response).response

    async def remove_stickers_from_favorite(
        self,
        sticker_ids: list[int],
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `store.removeStickersFromFavorite()`

        :param sticker_ids: Sticker IDs to be removed
        """

        params = self.get_set_params(locals())
        response = await self.api.request("store.removeStickersFromFavorite", params)
        model = BaseOkResponse
        return model(**response).response


__all__ = ("StoreCategory",)



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import FaveBookmark, FavePage, FaveTag, GroupsGroup, UsersUserFull
from vkbottle_types.responses.base_response import BaseResponse


class FaveAddTagResponse(BaseResponse):
    response: "FaveTag" = Field()


class FaveGetPagesResponseModel(BaseModel):
    count: int = Field()
    items: list["FavePage"] = Field()


class FaveGetPagesResponse(BaseResponse):
    response: "FaveGetPagesResponseModel" = Field()


class FaveGetTagsResponseModel(BaseModel):
    count: int = Field()
    items: list["FaveTag"] = Field()


class FaveGetTagsResponse(BaseResponse):
    response: "FaveGetTagsResponseModel" = Field()


class FaveGetExtendedResponseModel(BaseModel):
    count: int = Field()
    items: list["FaveBookmark"] = Field()
    profiles: list["UsersUserFull"] | None = Field(
        default=None,
    )
    groups: list["GroupsGroup"] | None = Field(
        default=None,
    )


class FaveGetExtendedResponse(BaseResponse):
    response: "FaveGetExtendedResponseModel" = Field()


class FaveGetResponseModel(BaseModel):
    count: int = Field()
    items: list["FaveBookmark"] = Field()


class FaveGetResponse(BaseResponse):
    response: "FaveGetResponseModel" = Field()



import typing

from vkbottle_types.codegen.methods.friends import FriendsCategory as _FriendsCategory  # type: ignore
from vkbottle_types.objects import (
    FriendsMutualFriend,
    FriendsOnlineUsers,
    FriendsOnlineUsersWithMobile,
    UsersFields,
)
from vkbottle_types.responses.friends import (
    FriendsGetMutualResponse,
    FriendsGetMutualTargetUidsResponse,
    FriendsGetMutualTotalCountResponse,
    FriendsGetOnlineExtendedResponse,
    FriendsGetOnlineOnlineMobileExtendedResponse,
    FriendsGetOnlineOnlineMobileResponse,
    FriendsGetOnlineOnlineMobileResponseModel,
    FriendsGetOnlineResponse,
    FriendsGetRequestsExtendedResponse,
    FriendsGetRequestsExtendedResponseModel,
    FriendsGetRequestsNeedMutualResponse,
    FriendsGetRequestsNeedMutualResponseModel,
    FriendsGetRequestsResponse,
    FriendsGetRequestsResponseModel,
    FriendsMutualFriend,
    FriendsOnlineUsers,
    FriendsOnlineUsersWithMobile,
)

if typing.TYPE_CHECKING:
    from vkbottle import ABCAPI  # type: ignore


class FriendsCategory(_FriendsCategory):  # type: ignore
    def __init__(self, api: "ABCAPI") -> None:
        super().__init__(api)

    @typing.overload
    async def get_mutual(
        self,
        *,
        total_count: int,
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
    ) -> "FriendsMutualFriend": ...

    @typing.overload
    async def get_mutual(
        self,
        *,
        target_uids: list[int],
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
    ) -> list[FriendsMutualFriend]: ...

    @typing.overload
    async def get_mutual(
        self,
        *,
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
    ) -> list[int]: ...

    async def get_mutual(
        self,
        *,
        target_uids: list[int] | None = None,
        total_count: int | None = None,
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsMutualFriend | list[int] | list[FriendsMutualFriend]:
        """Method `friends.getMutual()`

        :param target_uids: IDs of the users whose friends will be checked against the friends of the user specified in 'source_uid'.
        :param count: Number of mutual friends to return.
        :param need_common_count: Return mutual friends total count
        :param offset: Offset needed to return a specific subset of mutual friends.
        :param order: Sort order: 'random' - random order
        :param source_uid: ID of the user whose friends will be checked against the friends of the user specified in 'target_uid'.
        :param target_uid: ID of the user whose friends will be checked against the friends of the user specified in 'source_uid'.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getMutual", params)
        model = self.get_model(
            (
                (("target_uids",), FriendsGetMutualTargetUidsResponse),
                (("total_count",), FriendsGetMutualTotalCountResponse),
            ),
            default=FriendsGetMutualResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_online(
        self,
        *,
        online_mobile: typing.Literal[True],
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
    ) -> FriendsGetOnlineOnlineMobileResponseModel: ...

    @typing.overload
    async def get_online(
        self,
        *,
        extended: typing.Literal[True],
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
    ) -> "FriendsOnlineUsers": ...

    @typing.overload
    async def get_online(
        self,
        *,
        online_mobile_extended: typing.Literal[True],
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        online_mobile: bool | None = None,
        order: str | None = None,
        user_id: int | None = None,
    ) -> "FriendsOnlineUsersWithMobile": ...

    @typing.overload
    async def get_online(
        self,
        *,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
    ) -> list[int]: ...

    async def get_online(
        self,
        *,
        online_mobile: bool | None = None,
        extended: bool | None = None,
        online_mobile_extended: bool | None = None,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsOnlineUsersWithMobile | list[int] | FriendsGetOnlineOnlineMobileResponseModel | FriendsOnlineUsers:
        """Method `friends.getOnline()`

        :param online_mobile: '1' - to return an additional 'online_mobile' field, '0' - (default),
        :param count: Number of friends to return.
        :param list_id: Friend list ID. If this parameter is not set, information about all online friends is returned.
        :param offset: Offset needed to return a specific subset of friends.
        :param order: Sort order: 'random' - random order
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getOnline", params)
        model = self.get_model(
            (
                (("online_mobile",), FriendsGetOnlineOnlineMobileResponse),
                (("extended",), FriendsGetOnlineExtendedResponse),
                (("online_mobile_extended",), FriendsGetOnlineOnlineMobileExtendedResponse),
            ),
            default=FriendsGetOnlineResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_requests(
        self,
        *,
        need_mutual: typing.Literal[True],
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
    ) -> FriendsGetRequestsNeedMutualResponseModel: ...

    @typing.overload
    async def get_requests(
        self,
        *,
        extended: typing.Literal[True],
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
    ) -> FriendsGetRequestsExtendedResponseModel: ...

    @typing.overload
    async def get_requests(
        self,
        *,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
    ) -> FriendsGetRequestsResponseModel: ...

    async def get_requests(
        self,
        *,
        need_mutual: bool | None = None,
        extended: bool | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetRequestsNeedMutualResponseModel | FriendsGetRequestsResponseModel | FriendsGetRequestsExtendedResponseModel:
        """Method `friends.getRequests()`

        :param need_mutual: '1' - to return a list of mutual friends (up to 20), if any
        :param extended: '1' - to return response messages from users who have sent a friend request or, if 'suggested' is set to '1', to return a list of suggested friends
        :param count: Number of friend requests to return (default 100, maximum 1000).
        :param fields:
        :param need_viewed:
        :param offset: Offset needed to return a specific subset of friend requests.
        :param out: '1' - to return outgoing requests, '0' - to return incoming requests (default)
        :param ref:
        :param sort: Sort order: '1' - by number of mutual friends, '0' - by date
        :param suggested: '1' - to return a list of suggested friends, '0' - to return friend requests (default)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getRequests", params)
        model = self.get_model(
            (
                (("need_mutual",), FriendsGetRequestsNeedMutualResponse),
                (("extended",), FriendsGetRequestsExtendedResponse),
            ),
            default=FriendsGetRequestsResponse,
            params=params,
        )
        return model(**response).response



import typing

from vkbottle_types.events import Event, GroupEventType, GroupTypes, UserEventType, UserTypes
from vkbottle_types.events.bot_events import BaseGroupEvent
from vkbottle_types.events.user_events import BaseUserEvent

API_URL: typing.Final = "https://api.vk.ru/method/"
API_VERSION: typing.Final = "5.199"


__all__ = (
    "API_URL",
    "API_VERSION",
    "BaseGroupEvent",
    "BaseUserEvent",
    "Event",
    "GroupEventType",
    "GroupTypes",
    "UserEventType",
    "UserTypes",
)



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.responses.base_response import BaseResponse


class DownloadedGamesPaidStatusResponseModel(BaseModel):
    is_paid: bool = Field()


class DownloadedGamesPaidStatusResponse(BaseResponse):
    response: "DownloadedGamesPaidStatusResponseModel" = Field()



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import (
    AdsAccount,
    AdsAd,
    AdsAdLayout,
    AdsCampaign,
    AdsClient,
    AdsCreateAdStatus,
    AdsCreateCampaignStatus,
    AdsCreateClientsStatus,
    AdsDemoStats,
    AdsFloodStats,
    AdsLinkStatus,
    AdsPromotedPostReach,
    AdsRejectReason,
    AdsStats,
    AdsTargetGroup,
    AdsTargetPixelInfo,
    AdsTargSettings,
    AdsTargStats,
    AdsTargSuggestions,
    AdsTargSuggestionsCities,
    AdsTargSuggestionsRegions,
    AdsTargSuggestionsSchools,
    AdsUpdateAdsStatus,
    AdsUpdateClientsStatus,
    AdsUpdateOfficeUsersResult,
    AdsUsers,
)
from vkbottle_types.responses.ads import *  # noqa: F401,F403  # type: ignore
from vkbottle_types.responses.base import (
    BaseOkResponse,
    BaseOkResponseModel,
    BaseUndefinedResponse,
)


class AdsCategory(BaseCategory):
    async def add_office_users(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[bool]:
        """Method `ads.addOfficeUsers()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe added managers. Description of 'user_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.addOfficeUsers", params)
        model = AdsAddOfficeUsersResponse
        return model(**response).response

    async def check_link(
        self,
        account_id: int,
        link_type: str,
        link_url: str,
        campaign_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "AdsLinkStatus":
        """Method `ads.checkLink()`

        :param account_id: Advertising account ID.
        :param link_type: Object type: *'community' - community,, *'post' - community post,, *'application' - VK application,, *'video' - video,, *'site' - external site.
        :param link_url: Object URL.
        :param campaign_id: Campaign ID
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.checkLink", params)
        model = AdsCheckLinkResponse
        return model(**response).response

    async def create_ads(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsCreateAdStatus]:
        """Method `ads.createAds()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created ads. Description of 'ad_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createAds", params)
        model = AdsCreateAdsResponse
        return model(**response).response

    async def create_campaigns(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsCreateCampaignStatus]:
        """Method `ads.createCampaigns()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'campaign_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createCampaigns", params)
        model = AdsCreateCampaignsResponse
        return model(**response).response

    async def create_clients(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsCreateClientsStatus]:
        """Method `ads.createClients()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'client_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createClients", params)
        model = AdsCreateClientsResponse
        return model(**response).response

    async def create_lookalike_request(
        self,
        account_id: int,
        source_type: str,
        client_id: int | None = None,
        retargeting_group_id: int | None = None,
        **kwargs: typing.Any,
    ) -> AdsCreateLookalikeRequestResponseModel:
        """Method `ads.createLookalikeRequest()`

        :param account_id:
        :param source_type:
        :param client_id:
        :param retargeting_group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createLookalikeRequest", params)
        model = AdsCreateLookalikeRequestResponse
        return model(**response).response

    async def create_target_group(
        self,
        account_id: int,
        lifetime: int,
        name: str,
        client_id: int | None = None,
        target_pixel_id: int | None = None,
        target_pixel_rules: str | None = None,
        **kwargs: typing.Any,
    ) -> AdsCreateTargetGroupResponseModel:
        """Method `ads.createTargetGroup()`

        :param account_id: Advertising account ID.
        :param lifetime: 'For groups with auditory created with pixel code only.', , Number of days after that users will be automatically removed from the group.
        :param name: Name of the target group - a string up to 64 characters long.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param target_pixel_id:
        :param target_pixel_rules:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createTargetGroup", params)
        model = AdsCreateTargetGroupResponse
        return model(**response).response

    async def create_target_pixel(
        self,
        account_id: int,
        category_id: int,
        name: str,
        client_id: int | None = None,
        domain: str | None = None,
        **kwargs: typing.Any,
    ) -> AdsCreateTargetPixelResponseModel:
        """Method `ads.createTargetPixel()`

        :param account_id:
        :param category_id:
        :param name:
        :param client_id:
        :param domain:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createTargetPixel", params)
        model = AdsCreateTargetPixelResponse
        return model(**response).response

    async def delete_ads(
        self,
        account_id: int,
        ids: str,
        **kwargs: typing.Any,
    ) -> list[int]:
        """Method `ads.deleteAds()`

        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with ad IDs.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteAds", params)
        model = AdsDeleteAdsResponse
        return model(**response).response

    async def delete_campaigns(
        self,
        account_id: int,
        ids: str,
        **kwargs: typing.Any,
    ) -> list[int]:
        """Method `ads.deleteCampaigns()`

        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted campaigns.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteCampaigns", params)
        model = AdsDeleteCampaignsResponse
        return model(**response).response

    async def delete_clients(
        self,
        account_id: int,
        ids: str,
        **kwargs: typing.Any,
    ) -> list[int]:
        """Method `ads.deleteClients()`

        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted clients.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteClients", params)
        model = AdsDeleteClientsResponse
        return model(**response).response

    async def delete_target_group(
        self,
        account_id: int,
        target_group_id: int,
        client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `ads.deleteTargetGroup()`

        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteTargetGroup", params)
        model = BaseOkResponse
        return model(**response).response

    async def delete_target_pixel(
        self,
        account_id: int,
        target_pixel_id: int,
        client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> dict[str, typing.Any]:
        """Method `ads.deleteTargetPixel()`

        :param account_id:
        :param target_pixel_id:
        :param client_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteTargetPixel", params)
        model = BaseUndefinedResponse
        return model(**response).response

    async def get_accounts(
        self,
        **kwargs: typing.Any,
    ) -> list[AdsAccount]:
        """Method `ads.getAccounts()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAccounts", params)
        model = AdsGetAccountsResponse
        return model(**response).response

    async def get_ads(
        self,
        account_id: int,
        ad_ids: str | None = None,
        campaign_ids: str | None = None,
        client_id: int | None = None,
        include_deleted: bool | None = None,
        limit: int | None = None,
        offset: int | None = None,
        only_deleted: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsAd]:
        """Method `ads.getAds()`

        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'Available and required for advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown: *0 - show only active ads,, *1 - show all ads.
        :param limit: Limit of number of returned ads. Used only if ad_ids parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        :param only_deleted: Flag that specifies whether to show only archived ads: *0 - show all ads,, *1 - show only archived ads. Available when include_deleted flag is *1
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAds", params)
        model = AdsGetAdsResponse
        return model(**response).response

    async def get_ads_layout(
        self,
        account_id: int,
        ad_ids: str | None = None,
        campaign_ids: str | None = None,
        client_id: int | None = None,
        include_deleted: bool | None = None,
        limit: int | None = None,
        offset: int | None = None,
        only_deleted: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsAdLayout]:
        """Method `ads.getAdsLayout()`

        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 - show only active ads,, *1 - show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        :param only_deleted: Flag that specifies whether to show only archived ads: *0 - show all ads,, *1 - show only archived ads. Available when include_deleted flag is *1
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAdsLayout", params)
        model = AdsGetAdsLayoutResponse
        return model(**response).response

    async def get_ads_targeting(
        self,
        account_id: int,
        ad_ids: str | None = None,
        campaign_ids: str | None = None,
        client_id: int | None = None,
        include_deleted: bool | None = None,
        limit: int | None = None,
        offset: int | None = None,
        only_deleted: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSettings]:
        """Method `ads.getAdsTargeting()`

        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: flag that specifies whether archived ads shall be shown: *0 - show only active ads,, *1 - show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset needed to return a specific subset of results.
        :param only_deleted:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAdsTargeting", params)
        model = AdsGetAdsTargetingResponse
        return model(**response).response

    async def get_budget(
        self,
        account_id: int,
        **kwargs: typing.Any,
    ) -> str:
        """Method `ads.getBudget()`

        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getBudget", params)
        model = AdsGetBudgetResponse
        return model(**response).response

    async def get_campaigns(
        self,
        account_id: int,
        campaign_ids: str | None = None,
        client_id: int | None = None,
        fields: list[typing.Literal["ads_count"]] | None = None,
        include_deleted: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsCampaign]:
        """Method `ads.getCampaigns()`

        :param account_id: Advertising account ID.
        :param campaign_ids: Filter of advertising campaigns to show. Serialized JSON array with campaign IDs. Only campaigns that exist in 'campaign_ids' and belong to the specified advertising account will be shown. If the parameter is null, all campaigns will be shown.
        :param client_id: 'For advertising agencies'. ID of the client advertising campaigns are retrieved from.
        :param fields:
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 - show only active campaigns,, *1 - show all campaigns.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getCampaigns", params)
        model = AdsGetCampaignsResponse
        return model(**response).response

    async def get_categories(
        self,
        lang: str | None = None,
        **kwargs: typing.Any,
    ) -> AdsGetCategoriesResponseModel:
        """Method `ads.getCategories()`

        :param lang: Language. The full list of supported languages is [vk.com/dev/api_requests|here].
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getCategories", params)
        model = AdsGetCategoriesResponse
        return model(**response).response

    async def get_clients(
        self,
        account_id: int,
        **kwargs: typing.Any,
    ) -> list[AdsClient]:
        """Method `ads.getClients()`

        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getClients", params)
        model = AdsGetClientsResponse
        return model(**response).response

    async def get_demographics(
        self,
        account_id: int,
        date_from: str,
        date_to: str,
        ids: str,
        ids_type: str,
        period: str,
        **kwargs: typing.Any,
    ) -> list[AdsDemoStats]:
        """Method `ads.getDemographics()`

        :param account_id: Advertising account ID.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 - September 27, 2011, **0 - day it was created on,, *month: YYYY-MM, example: 2011-09 - September 2011, **0 - month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 - September 27, 2011, **0 - current day,, *month: YYYY-MM, example: 2011-09 - September 2011, **0 - current month,, *overall: 0.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad - ads,, *campaign - campaigns.
        :param period: Data grouping by dates: *day - statistics by days,, *month - statistics by months,, *overall - overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getDemographics", params)
        model = AdsGetDemographicsResponse
        return model(**response).response

    async def get_flood_stats(
        self,
        account_id: int,
        **kwargs: typing.Any,
    ) -> "AdsFloodStats":
        """Method `ads.getFloodStats()`

        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getFloodStats", params)
        model = AdsGetFloodStatsResponse
        return model(**response).response

    async def get_lookalike_requests(
        self,
        account_id: int,
        client_id: int | None = None,
        limit: int | None = None,
        offset: int | None = None,
        requests_ids: str | None = None,
        sort_by: str | None = None,
        **kwargs: typing.Any,
    ) -> AdsGetLookalikeRequestsResponseModel:
        """Method `ads.getLookalikeRequests()`

        :param account_id:
        :param client_id:
        :param limit:
        :param offset:
        :param requests_ids:
        :param sort_by:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getLookalikeRequests", params)
        model = AdsGetLookalikeRequestsResponse
        return model(**response).response

    async def get_musicians(
        self,
        artist_name: str,
        **kwargs: typing.Any,
    ) -> AdsGetMusiciansResponseModel:
        """Method `ads.getMusicians()`

        :param artist_name:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getMusicians", params)
        model = AdsGetMusiciansResponse
        return model(**response).response

    async def get_musicians_by_ids(
        self,
        ids: list[int],
        **kwargs: typing.Any,
    ) -> AdsGetMusiciansResponseModel:
        """Method `ads.getMusiciansByIds()`

        :param ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getMusiciansByIds", params)
        model = AdsGetMusiciansResponse
        return model(**response).response

    async def get_office_users(
        self,
        account_id: int,
        **kwargs: typing.Any,
    ) -> list[AdsUsers]:
        """Method `ads.getOfficeUsers()`

        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getOfficeUsers", params)
        model = AdsGetOfficeUsersResponse
        return model(**response).response

    async def get_posts_reach(
        self,
        account_id: int,
        ids: str,
        ids_type: str,
        **kwargs: typing.Any,
    ) -> list[AdsPromotedPostReach]:
        """Method `ads.getPostsReach()`

        :param account_id: Advertising account ID.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 100 objects.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad - ads,, *campaign - campaigns.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getPostsReach", params)
        model = AdsGetPostsReachResponse
        return model(**response).response

    async def get_rejection_reason(
        self,
        account_id: int,
        ad_id: int,
        **kwargs: typing.Any,
    ) -> "AdsRejectReason":
        """Method `ads.getRejectionReason()`

        :param account_id: Advertising account ID.
        :param ad_id: Ad ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getRejectionReason", params)
        model = AdsGetRejectionReasonResponse
        return model(**response).response

    async def get_statistics(
        self,
        account_id: int,
        date_from: str,
        date_to: str,
        ids: str,
        ids_type: str,
        period: str,
        stats_fields: list[typing.Literal["views_times"]] | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsStats]:
        """Method `ads.getStatistics()`

        :param account_id: Advertising account ID.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 - September 27, 2011, **0 - day it was created on,, *month: YYYY-MM, example: 2011-09 - September 2011, **0 - month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 - September 27, 2011, **0 - current day,, *month: YYYY-MM, example: 2011-09 - September 2011, **0 - current month,, *overall: 0.
        :param ids: IDs requested ads, campaigns, clients or account, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad - ads,, *campaign - campaigns,, *client - clients,, *office - account.
        :param period: Data grouping by dates: *day - statistics by days,, *month - statistics by months,, *overall - overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param stats_fields: Additional fields to add to statistics
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getStatistics", params)
        model = AdsGetStatisticsResponse
        return model(**response).response

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        cities: str,
        country: int | None = None,
        ids: str | None = None,
        lang: str | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSuggestionsRegions]: ...

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        cities: str | None = None,
        country: int | None = None,
        ids: str | None = None,
        lang: str | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSuggestionsCities]: ...

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        cities: str,
        country: int | None = None,
        ids: str | None = None,
        lang: str | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSuggestionsSchools]: ...

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        cities: str | None = None,
        country: int | None = None,
        ids: str | None = None,
        lang: str | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSuggestions]: ...

    async def get_suggestions(
        self,
        section: str,
        cities: str | None = None,
        country: int | None = None,
        ids: str | None = None,
        lang: str | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSuggestionsCities] | list[AdsTargSuggestions] | list[AdsTargSuggestionsRegions] | list[AdsTargSuggestionsSchools]:
        """Method `ads.getSuggestions()`

        :param section: Section, suggestions are retrieved in. Available values: *countries - request of a list of countries. If q is not set or blank, a short list of countries is shown. Otherwise, a full list of countries is shown. *regions - requested list of regions. 'country' parameter is required. *cities - requested list of cities. 'country' parameter is required. *districts - requested list of districts. 'cities' parameter is required. *stations - requested list of subway stations. 'cities' parameter is required. *streets - requested list of streets. 'cities' parameter is required. *schools - requested list of educational organizations. 'cities' parameter is required. *interests - requested list of interests. *positions - requested list of positions (professions). *group_types - requested list of group types. *religions - requested list of religious commitments. *browsers - requested list of browsers and mobile devices.
        :param cities: IDs of cities where objects are searched in, separated with a comma.
        :param country: ID of the country objects are searched in.
        :param ids: Objects IDs separated by commas. If the parameter is passed, 'q, country, cities' should not be passed.
        :param lang: Language of the returned string values. Supported languages: *ru - Russian,, *ua - Ukrainian,, *en - English.
        :param q: Filter-line of the request (for countries, regions, cities, streets, schools, interests, positions).
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getSuggestions", params)
        model = self.get_model(
            (
                (("regions",), AdsGetSuggestionsRegionsResponse),
                (("cities",), AdsGetSuggestionsRegionsResponse),
                (("schools",), AdsGetSuggestionsSchoolsResponse),
            ),
            default=AdsGetSuggestionsResponse,
            params=params,
        )
        return model(**response).response

    async def get_target_groups(
        self,
        account_id: int,
        client_id: int | None = None,
        extended: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargetGroup]:
        """Method `ads.getTargetGroups()`

        :param account_id: Advertising account ID.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param extended: '1' - to return pixel code.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getTargetGroups", params)
        model = AdsGetTargetGroupsResponse
        return model(**response).response

    async def get_target_pixels(
        self,
        account_id: int,
        client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargetPixelInfo]:
        """Method `ads.getTargetPixels()`

        :param account_id:
        :param client_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getTargetPixels", params)
        model = AdsGetTargetPixelsResponse
        return model(**response).response

    async def get_targeting_stats(
        self,
        account_id: int,
        link_url: str,
        ad_format: int | None = None,
        ad_id: int | None = None,
        ad_platform: str | None = None,
        ad_platform_no_ad_network: str | None = None,
        ad_platform_no_wall: str | None = None,
        client_id: int | None = None,
        criteria: str | None = None,
        impressions_limit_period: int | None = None,
        link_domain: str | None = None,
        need_precise: bool | None = None,
        publisher_platforms: str | None = None,
        **kwargs: typing.Any,
    ) -> "AdsTargStats":
        """Method `ads.getTargetingStats()`

        :param account_id: Advertising account ID.
        :param link_url: URL for the advertised object.
        :param ad_format: Ad format. Possible values: *'1' - image and text,, *'2' - big image,, *'3' - exclusive format,, *'4' - community, square image,, *'7' - special app format,, *'8' - special community format,, *'9' - post in community,, *'10' - app board.
        :param ad_id: ID of an ad which targeting parameters shall be analyzed.
        :param ad_platform: Platforms to use for ad showing. Possible values: (for 'ad_format' = '1'), *'0' - VK and partner sites,, *'1' - VK only. (for 'ad_format' = '9'), *'all' - all platforms,, *'desktop' - desktop version,, *'mobile' - mobile version and apps.
        :param ad_platform_no_ad_network:
        :param ad_platform_no_wall:
        :param client_id:
        :param criteria: Serialized JSON object that describes targeting parameters. Description of 'criteria' object see below.
        :param impressions_limit_period: Impressions limit period in seconds, must be a multiple of 86400(day)
        :param link_domain: Domain of the advertised object.
        :param need_precise: Additionally return recommended cpc and cpm to reach 5,10..95 percents of audience.
        :param publisher_platforms:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getTargetingStats", params)
        model = AdsGetTargetingStatsResponse
        return model(**response).response

    async def get_upload_url(
        self,
        ad_format: int,
        icon: int | None = None,
        **kwargs: typing.Any,
    ) -> str:
        """Method `ads.getUploadURL()`

        :param ad_format: Ad format: *1 - image and text,, *2 - big image,, *3 - exclusive format,, *4 - community, square image,, *7 - special app format.
        :param icon:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getUploadURL", params)
        model = AdsGetUploadURLResponse
        return model(**response).response

    async def get_video_upload_url(
        self,
        **kwargs: typing.Any,
    ) -> str:
        """Method `ads.getVideoUploadURL()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getVideoUploadURL", params)
        model = AdsGetVideoUploadURLResponse
        return model(**response).response

    async def import_target_contacts(
        self,
        account_id: int,
        contacts: str,
        target_group_id: int,
        client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> int:
        """Method `ads.importTargetContacts()`

        :param account_id: Advertising account ID.
        :param contacts: List of phone numbers, emails or user IDs separated with a comma.
        :param target_group_id: Target group ID.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.importTargetContacts", params)
        model = AdsImportTargetContactsResponse
        return model(**response).response

    async def remove_office_users(
        self,
        account_id: int,
        ids: str,
        **kwargs: typing.Any,
    ) -> list[bool]:
        """Method `ads.removeOfficeUsers()`

        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted managers.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.removeOfficeUsers", params)
        model = AdsRemoveOfficeUsersResponse
        return model(**response).response

    async def remove_target_contacts(
        self,
        account_id: int,
        contacts: str,
        target_group_id: int,
        client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> AdsRemoveTargetContactsResponseModel:
        """Method `ads.removeTargetContacts()`

        :param account_id:
        :param contacts:
        :param target_group_id:
        :param client_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.removeTargetContacts", params)
        model = AdsRemoveTargetContactsResponse
        return model(**response).response

    async def save_lookalike_request_result(
        self,
        account_id: int,
        level: int,
        request_id: int,
        client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> AdsSaveLookalikeRequestResultResponseModel:
        """Method `ads.saveLookalikeRequestResult()`

        :param account_id:
        :param level:
        :param request_id:
        :param client_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.saveLookalikeRequestResult", params)
        model = AdsSaveLookalikeRequestResultResponse
        return model(**response).response

    async def share_target_group(
        self,
        account_id: int,
        target_group_id: int,
        client_id: int | None = None,
        share_with_client_id: int | None = None,
        **kwargs: typing.Any,
    ) -> AdsShareTargetGroupResponseModel:
        """Method `ads.shareTargetGroup()`

        :param account_id:
        :param target_group_id:
        :param client_id:
        :param share_with_client_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.shareTargetGroup", params)
        model = AdsShareTargetGroupResponse
        return model(**response).response

    async def update_ads(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsUpdateAdsStatus]:
        """Method `ads.updateAds()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in ads. Description of 'ad_edit_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateAds", params)
        model = AdsUpdateAdsResponse
        return model(**response).response

    async def update_campaigns(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsCreateCampaignStatus]:
        """Method `ads.updateCampaigns()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in campaigns. Description of 'campaign_mod' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateCampaigns", params)
        model = AdsUpdateCampaignsResponse
        return model(**response).response

    async def update_clients(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsUpdateClientsStatus]:
        """Method `ads.updateClients()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in clients. Description of 'client_mod' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateClients", params)
        model = AdsUpdateClientsResponse
        return model(**response).response

    async def update_office_users(
        self,
        account_id: int,
        data: str,
        **kwargs: typing.Any,
    ) -> list[AdsUpdateOfficeUsersResult]:
        """Method `ads.updateOfficeUsers()`

        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe added managers. Description of 'user_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateOfficeUsers", params)
        model = AdsUpdateOfficeUsersResponse
        return model(**response).response

    async def update_target_group(
        self,
        account_id: int,
        lifetime: int,
        name: str,
        target_group_id: int,
        client_id: int | None = None,
        domain: str | None = None,
        target_pixel_id: int | None = None,
        target_pixel_rules: str | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `ads.updateTargetGroup()`

        :param account_id: Advertising account ID.
        :param lifetime: 'Only for the groups that get audience from sites with user accounting code.', Time in days when users added to a retarget group will be automatically excluded from it. '0' - automatic exclusion is off.
        :param name: New name of the target group - a string up to 64 characters long.
        :param target_group_id: Group ID.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        :param domain: Domain of the site where user accounting code will be placed.
        :param target_pixel_id:
        :param target_pixel_rules:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateTargetGroup", params)
        model = BaseOkResponse
        return model(**response).response

    async def update_target_pixel(
        self,
        account_id: int,
        category_id: int,
        name: str,
        target_pixel_id: int,
        client_id: int | None = None,
        domain: str | None = None,
        **kwargs: typing.Any,
    ) -> dict[str, typing.Any]:
        """Method `ads.updateTargetPixel()`

        :param account_id:
        :param category_id:
        :param name:
        :param target_pixel_id:
        :param client_id:
        :param domain:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateTargetPixel", params)
        model = BaseUndefinedResponse
        return model(**response).response


__all__ = ("AdsCategory",)



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import LeadFormsForm
from vkbottle_types.responses.lead_forms import *  # noqa: F401,F403  # type: ignore


class LeadFormsCategory(BaseCategory):
    async def create(
        self,
        description: str,
        group_id: int,
        name: str,
        policy_link_url: str,
        questions: str,
        title: str,
        active: bool | None = None,
        confirmation: str | None = None,
        notify_admins: list[int] | None = None,
        notify_emails: list[str] | None = None,
        once_per_user: bool | None = None,
        photo: str | None = None,
        pixel_code: str | None = None,
        site_link_url: str | None = None,
        **kwargs: typing.Any,
    ) -> LeadFormsCreateResponseModel:
        """Method `leadForms.create()`

        :param description:
        :param group_id:
        :param name:
        :param policy_link_url:
        :param questions:
        :param title:
        :param active:
        :param confirmation:
        :param notify_admins:
        :param notify_emails:
        :param once_per_user:
        :param photo:
        :param pixel_code:
        :param site_link_url:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.create", params)
        model = LeadFormsCreateResponse
        return model(**response).response

    async def delete(
        self,
        form_id: int,
        group_id: int,
        **kwargs: typing.Any,
    ) -> LeadFormsDeleteResponseModel:
        """Method `leadForms.delete()`

        :param form_id:
        :param group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.delete", params)
        model = LeadFormsDeleteResponse
        return model(**response).response

    async def get(
        self,
        form_id: int,
        group_id: int,
        **kwargs: typing.Any,
    ) -> "LeadFormsForm":
        """Method `leadForms.get()`

        :param form_id:
        :param group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.get", params)
        model = LeadFormsGetResponse
        return model(**response).response

    async def get_leads(
        self,
        form_id: int,
        group_id: int,
        limit: int | None = None,
        next_page_token: str | None = None,
        **kwargs: typing.Any,
    ) -> LeadFormsGetLeadsResponseModel:
        """Method `leadForms.getLeads()`

        :param form_id:
        :param group_id:
        :param limit:
        :param next_page_token:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.getLeads", params)
        model = LeadFormsGetLeadsResponse
        return model(**response).response

    async def get_upload_url(
        self,
        **kwargs: typing.Any,
    ) -> str:
        """Method `leadForms.getUploadURL()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.getUploadURL", params)
        model = LeadFormsUploadUrlResponse
        return model(**response).response

    async def get_list(
        self,
        group_id: int,
        **kwargs: typing.Any,
    ) -> list[LeadFormsForm]:
        """Method `leadForms.list()`

        :param group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.list", params)
        model = LeadFormsListResponse
        return model(**response).response

    async def update(
        self,
        description: str,
        form_id: int,
        group_id: int,
        name: str,
        policy_link_url: str,
        questions: str,
        title: str,
        active: bool | None = None,
        confirmation: str | None = None,
        notify_admins: list[int] | None = None,
        notify_emails: list[str] | None = None,
        once_per_user: bool | None = None,
        photo: str | None = None,
        pixel_code: str | None = None,
        site_link_url: str | None = None,
        **kwargs: typing.Any,
    ) -> LeadFormsCreateResponseModel:
        """Method `leadForms.update()`

        :param description:
        :param form_id:
        :param group_id:
        :param name:
        :param policy_link_url:
        :param questions:
        :param title:
        :param active:
        :param confirmation:
        :param notify_admins:
        :param notify_emails:
        :param once_per_user:
        :param photo:
        :param pixel_code:
        :param site_link_url:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.update", params)
        model = LeadFormsCreateResponse
        return model(**response).response


__all__ = ("LeadFormsCategory",)
